/* -*- c++ -*- ***************************************************************/
/* Echtzeitsysteme                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*                            C o n s o l e                                  */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/* Printing text on the GBA screen                                           */
/*---------------------------------------------------------------------------*/

#include "devices/console.h"

#define GBA_BASE_PAL_RAM	(unsigned short *)0x5000000
#define GBA_BASE_VRAM		(unsigned short *)0x6000000

extern const unsigned char fontData[];

void Console::init() {
  unsigned short i, j, k;
  unsigned char temp8;
  unsigned short *palette = GBA_BASE_PAL_RAM;
  unsigned short *tiles = GBA_BASE_VRAM;
  unsigned char map[6];

  map[0] = 0;
  map[1] = 6;
  map[2] = 12;
  map[3] = 18;
  map[4] = 25;
  map[5] = 31;

  textMap = GBA_BASE_VRAM + 0x2000;
  DISPCNT = 0x0400;
  BG2CNT = 0xa880;
	
  // assume grayscale for now
  for (i=0; i<32; i++) {
    palette[i]= i|(i<<5)|(i<<10);
  }
	
  // the rest of the entries are "cube 6" colors (6*6*6 = 216)
  for (i=0; i<6; i++) {
    for (j=0; j<6; j++) {
      for (k=0; k<6; k++) {
	palette[i*36 + j*6 + k + 32] = (map[i]<<10)|(map[j]<<5)|(map[k]*6); 
      }
    }
  }
		
  for (i=0; i<32*32; i++) {
    tiles[i] = 0;
  }

  bgColor = 0;
  textColor = 31;

  for(i=32; i<128; i++) {
    for (j=0; j<8; j++) {
      temp8 = fontData[(i-32)*8 + j];
      tiles[i*32+j*4+0] = ((temp8&0x80)?textColor:bgColor) | ((temp8&0x40)?textColor<<8:bgColor<<8);
      tiles[i*32+j*4+1] = ((temp8&0x20)?textColor:bgColor) | ((temp8&0x10)?textColor<<8:bgColor<<8);
      tiles[i*32+j*4+2] = ((temp8&0x08)?textColor:bgColor) | ((temp8&0x04)?textColor<<8:bgColor<<8);
      tiles[i*32+j*4+3] = ((temp8&0x02)?textColor:bgColor) | ((temp8&0x01)?textColor<<8:bgColor<<8);
    }
  }

  for (i=128*32; i<(128+32)*32; i++) {
    tiles[i] = 0;
  }

  bgColor = 31;
  textColor = 0;

  for(i=128+32; i<256; i++) {
    for (j=0; j<8; j++) {
      temp8 = fontData[(i-128-32)*8 + j];
      tiles[i*32+j*4+0] = ((temp8&0x80)?textColor:bgColor) | ((temp8&0x40)?textColor<<8:bgColor<<8);
      tiles[i*32+j*4+1] = ((temp8&0x20)?textColor:bgColor) | ((temp8&0x10)?textColor<<8:bgColor<<8);
      tiles[i*32+j*4+2] = ((temp8&0x08)?textColor:bgColor) | ((temp8&0x04)?textColor<<8:bgColor<<8);
      tiles[i*32+j*4+3] = ((temp8&0x02)?textColor:bgColor) | ((temp8&0x01)?textColor<<8:bgColor<<8);
    }
  }

  clear();
}

void Console::crlf() {
  unsigned char x;
  unsigned short temp16;

  xPos = 0;
  yPos++;

  if ((yPos-scrollPos)==HEIGHT) {
    scroll(1);

    if (scrollPos>=VHEIGHT && yPos>=VHEIGHT) {
      scrollPos -= VHEIGHT;
      yPos -= VHEIGHT;
    }
  }

  // clear line
  temp16 = (yPos&(VHEIGHT-1))*VWIDTH; 
  for(x=0; x<WIDTH; x++) {
    textMap[temp16 + x] = 32;
  }
}

void Console::print(char c) {
  ezstubs_uint16 temp16;
  temp16 = (yPos&(VHEIGHT-1))*VWIDTH;

  if (xPos==WIDTH) {
    crlf();
    temp16 = (yPos&(VHEIGHT-1))*VWIDTH;
  }

  if (c==10) {
    crlf();
    temp16 = (yPos&(VHEIGHT-1))*VWIDTH;
  }
  else {
    textMap[temp16 + xPos] = (ezstubs_uint16)(c | orMask);
    xPos++;
  }
}

void Console::scroll(short val) {
  scrollPos += val;
  BG2VOFS = (scrollPos&(VHEIGHT-1))*8;
}

void Console::clear() {
  unsigned char x, y;

  xPos = 0;
  yPos = 0;
  orMask = 0;
  scrollPos = 0;
  scroll(0);

  for(y=0; y<HEIGHT; y++) {
    for(x=0; x<WIDTH; x++) {
      textMap[y*VWIDTH + x] = 32;
    }
  }
}

void Console::setProperty(DispProp property, short val) {
  switch (property) {
  case DP_REVERSE:
    if (val)
      orMask |= 0x0080;
    else
      orMask &= ~0x0080;
    break;
    //...
  }
}


void Console::setPosition(unsigned char x, unsigned char y) {
  xPos = x;
  yPos = y + scrollPos;
}

void Console::getPosition(unsigned char *x, unsigned char *y) {
  if (x) {
    *x = (unsigned char)xPos;
  }
  if (y) {
    *y = (unsigned char)((yPos - scrollPos)&(VHEIGHT-1));
  }
}


#if 1
const unsigned char fontData[]=
{	/* 32 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 33 */	0x10, 0x10, 0x10, 0x10, 0x10, 0x00, 0x10, 0x00,
	/* 34 */	0x28, 0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 35 */	0x28, 0x28, 0xfe, 0x28, 0xfe, 0x28, 0x28, 0x00,
	/* 36 */	0x10, 0x7c, 0x80, 0x7c, 0x02, 0x7c, 0x10, 0x00,
	/* 36 */	0xc2, 0xc4, 0x08, 0x10, 0x20, 0x46, 0x86, 0x00,
	/* 38 */	0x60, 0x90, 0x60, 0x90, 0x88, 0x84, 0x7a, 0x00,
	/* 39 */	0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 40 */	0x08, 0x10, 0x20, 0x20, 0x20, 0x10, 0x08, 0x00,
	/* 41 */	0x20, 0x10, 0x08, 0x08, 0x08, 0x10, 0x20, 0x00,
	/* 42 */	0x44, 0x28, 0x10, 0xfe, 0x10, 0x28, 0x44, 0x00,
	/* 43 */	0x00, 0x10, 0x10, 0x7c, 0x10, 0x10, 0x00, 0x00,
	/* 44 */	0x00, 0x00, 0x00, 0x00, 0x18, 0x08, 0x10, 0x00,
	/* 45 */	0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00, 0x00,
	/* 46 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00,
	/* 47 */	0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x00,
	/* 48 */	0x7c, 0x86, 0x8a, 0x92, 0xa2, 0xc2, 0x7c, 0x00,
	/* 49 */	0x10, 0x30, 0x10, 0x10, 0x10, 0x10, 0x38, 0x00,
	/* 50 */	0x7c, 0x82, 0x04, 0x08, 0x10, 0x20, 0xfe, 0x00,
	/* 51 */	0x7c, 0x82, 0x02, 0x3c, 0x02, 0x82, 0x7c, 0x00,
	/* 52 */	0x40, 0x40, 0x84, 0x84, 0xfe, 0x04, 0x04, 0x00,
	/* 53 */	0xfe, 0x80, 0x80, 0xfc, 0x02, 0x82, 0x7c, 0x00,
	/* 54 */	0x10, 0x20, 0x40, 0xfc, 0x82, 0x82, 0x7c, 0x00,
	/* 55 */	0xfe, 0x02, 0x04, 0x08, 0x10, 0x10, 0x10, 0x00,
	/* 56 */	0x7c, 0x82, 0x82, 0x7c, 0x82, 0x82, 0x7c, 0x00,
	/* 57 */	0x7c, 0x82, 0x82, 0x7e, 0x04, 0x08, 0x10, 0x00,
	/* 58 */	0x00, 0x00, 0x10, 0x00, 0x10, 0x00, 0x00, 0x00,
	/* 59 */	0x00, 0x00, 0x10, 0x00, 0x10, 0x10, 0x20, 0x00,
	/* 60 */	0x04, 0x08, 0x10, 0x20, 0x10, 0x08, 0x04, 0x00,
	/* 61 */	0x00, 0x00, 0x7c, 0x00, 0x7c, 0x00, 0x00, 0x00,
	/* 62 */	0x40, 0x20, 0x10, 0x08, 0x10, 0x20, 0x40, 0x00,
	/* 63 */	0x7c, 0x82, 0x04, 0x08, 0x10, 0x00, 0x10, 0x00,
	/* 64 */	0x7c, 0x82, 0xbe, 0xa2, 0xbe, 0x80, 0x7e, 0x00,
	/* 65 */	0x10, 0x28, 0x44, 0x82, 0xfe, 0x82, 0x82, 0x00,
	/* 66 */	0xfc, 0x82, 0x82, 0xfc, 0x82, 0x82, 0xfc, 0x00,
	/* 67 */	0x7c, 0x82, 0x80, 0x80, 0x80, 0x82, 0x7c, 0x00,
	/* 68 */	0xfc, 0x82, 0x82, 0x82, 0x82, 0x82, 0xfc, 0x00,
	/* 69 */	0xfe, 0x80, 0x80, 0xfc, 0x80, 0x80, 0xfe, 0x00,
	/* 70 */	0xfe, 0x80, 0x80, 0xfc, 0x80, 0x80, 0x80, 0x00,
	/* 71 */	0x7c, 0x82, 0x80, 0x8e, 0x82, 0x82, 0x7e, 0x00,
	/* 72 */	0x82, 0x82, 0x82, 0xfe, 0x82, 0x82, 0x82, 0x00,
	/* 73 */	0x38, 0x10, 0x10, 0x10, 0x10, 0x10, 0x38, 0x00,
	/* 64 */	0x02, 0x02, 0x02, 0x02, 0x02, 0x82, 0x7c, 0x00,
	/* 75 */	0x82, 0x8c, 0xb0, 0xc0, 0xb0, 0x8c, 0x82, 0x00,
	/* 76 */	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xfe, 0x00,
	/* 77 */	0x82, 0xc6, 0xaa, 0x92, 0x82, 0x82, 0x82, 0x00,
	/* 78 */	0x82, 0xc2, 0xa2, 0x92, 0x8a, 0x86, 0x82, 0x00,
	/* 79 */	0x7c, 0x82, 0x82, 0x82, 0x82, 0x82, 0x7c, 0x00,
	/* 80 */	0xfc, 0x82, 0x82, 0xfc, 0x80, 0x80, 0x80, 0x00,
	/* 81 */	0x7c, 0x82, 0x82, 0x82, 0x82, 0x84, 0x7a, 0x00,
	/* 82 */	0xfc, 0x82, 0x82, 0xfc, 0x88, 0x84, 0x82, 0x00,
	/* 83 */	0x7c, 0x82, 0x80, 0x7c, 0x02, 0x82, 0x7c, 0x00,
	/* 84 */	0xfe, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x00,
	/* 85 */	0x82, 0x82, 0x82, 0x82, 0x82, 0x82, 0x7c, 0x00,
	/* 86 */	0x82, 0x82, 0x82, 0x82, 0x44, 0x28, 0x10, 0x00,
	/* 87 */	0x82, 0x82, 0x82, 0x92, 0x92, 0x54, 0x28, 0x00,
	/* 88 */	0x82, 0x44, 0x28, 0x10, 0x28, 0x44, 0x82, 0x00,
	/* 89 */	0x82, 0x82, 0x44, 0x28, 0x10, 0x10, 0x10, 0x00,
	/* 90 */	0xfe, 0x04, 0x08, 0x10, 0x20, 0x40, 0xfe, 0x00,
	/* 91 */	0x38, 0x20, 0x20, 0x20, 0x20, 0x20, 0x38, 0x00,
	/* 92 */	0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x00,
	/* 93 */	0x38, 0x08, 0x08, 0x08, 0x08, 0x08, 0x38, 0x00,
	/* 94 */	0x10, 0x28, 0x44, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 95 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x00,
	/* 96 */	0x20, 0x10, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 97 */	0x00, 0x00, 0x7e, 0x82, 0x82, 0x86, 0x7a, 0x00,
	/* 98 */	0x80, 0x80, 0xfc, 0x82, 0x82, 0x82, 0xfc, 0x00,
	/* 99 */	0x00, 0x00, 0x7e, 0x80, 0x80, 0x80, 0x7e, 0x00,
	/* 100 */	0x02, 0x02, 0x7e, 0x82, 0x82, 0x82, 0x7e, 0x00,
	/* 101 */	0x00, 0x00, 0x7c, 0x82, 0xfe, 0x80, 0x7c, 0x00,
	/* 102 */	0x0e, 0x10, 0x7c, 0x10, 0x10, 0x10, 0x10, 0x00,
	/* 103 */	0x00, 0x00, 0x7e, 0x82, 0x7e, 0x02, 0x7c, 0x00,
	/* 104 */	0x80, 0x80, 0xfc, 0x82, 0x82, 0x82, 0x82, 0x00,
	/* 105 */	0x10, 0x00, 0x30, 0x10, 0x10, 0x10, 0x38, 0x00,
	/* 106 */	0x08, 0x00, 0x18, 0x08, 0x08, 0x08, 0x70, 0x00,
	/* 107 */	0x80, 0x80, 0x86, 0x98, 0xe0, 0x98, 0x86, 0x00,
	/* 108 */	0x30, 0x10, 0x10, 0x10, 0x10, 0x10, 0x38, 0x00,
	/* 109 */	0x00, 0x00, 0xec, 0x92, 0x92, 0x92, 0x82, 0x00,
	/* 110 */	0x00, 0x00, 0xbc, 0xc2, 0x82, 0x82, 0x82, 0x00,
	/* 111 */	0x00, 0x00, 0x7c, 0x82, 0x82, 0x82, 0x7c, 0x00,
	/* 112 */	0x00, 0x00, 0xfc, 0x82, 0xfc, 0x80, 0x80, 0x00,
	/* 113 */	0x00, 0x00, 0x7e, 0x82, 0x7e, 0x02, 0x02, 0x00,
	/* 114 */	0x00, 0x00, 0xbe, 0xc0, 0x80, 0x80, 0x80, 0x00,
	/* 115 */	0x00, 0x00, 0x7e, 0x80, 0x7c, 0x02, 0xfc, 0x00,
	/* 116 */	0x10, 0x10, 0x7c, 0x10, 0x10, 0x10, 0x0c, 0x00,
	/* 117 */	0x00, 0x00, 0x82, 0x82, 0x82, 0x86, 0x7a, 0x00,
	/* 118 */	0x00, 0x00, 0x82, 0x82, 0x44, 0x28, 0x10, 0x00,
	/* 119 */	0x00, 0x00, 0x92, 0x92, 0x92, 0x92, 0x6c, 0x00,
	/* 120 */	0x00, 0x00, 0x82, 0x44, 0x38, 0x44, 0x82, 0x00,
	/* 121 */	0x00, 0x00, 0x82, 0x82, 0x7e, 0x02, 0x7c, 0x00,
	/* 122 */	0x00, 0x00, 0xfe, 0x0c, 0x30, 0xc0, 0xfe, 0x00,
	/* 123 */	0x0c, 0x10, 0x10, 0x20, 0x10, 0x10, 0x0c, 0x00,
	/* 124 */	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x00,
	/* 125 */	0x60, 0x10, 0x10, 0x08, 0x10, 0x10, 0x60, 0x00,
	/* 126 */	0x34, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 127 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
#else
const unsigned char fontData[]=
{
	/* 32 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	/* 33 */	0x30, 0x78, 0x78, 0x30, 0x30, 0x00, 0x30, 0x00, 
	/* 34 */	0x6c, 0x6c, 0x6c, 0x00, 0x00, 0x00, 0x00, 0x00, 
	/* 35 */	0x6c, 0x6c, 0xfe, 0x6c, 0xfe, 0x6c, 0x6c, 0x00, 
	/* 36 */	0x30, 0x7c, 0xc0, 0x78, 0x0c, 0xf8, 0x30, 0x00, 
	/* 37 */	0x00, 0xc6, 0xcc, 0x18, 0x30, 0x66, 0xc6, 0x00, 
	/* 38 */	0x38, 0x6c, 0x38, 0x76, 0xdc, 0xcc, 0x76, 0x00, 
	/* 39 */	0x60, 0x60, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 
	/* 40 */	0x18, 0x30, 0x60, 0x60, 0x60, 0x30, 0x18, 0x00, 
	/* 41 */	0x60, 0x30, 0x18, 0x18, 0x18, 0x30, 0x60, 0x00, 
	/* 42 */	0x00, 0x66, 0x3c, 0xff, 0x3c, 0x66, 0x00, 0x00, 
	/* 43 */	0x00, 0x30, 0x30, 0xfc, 0x30, 0x30, 0x00, 0x00, 
	/* 44 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x30, 0x60, 
	/* 45 */	0x00, 0x00, 0x00, 0xfc, 0x00, 0x00, 0x00, 0x00, 
	/* 46 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x30, 0x00, 
	/* 47 */	0x06, 0x0c, 0x18, 0x30, 0x60, 0xc0, 0x80, 0x00, 
	/* 48 */	0x78, 0xcc, 0xdc, 0xfc, 0xec, 0xcc, 0x78, 0x00, 
	/* 49 */	0x30, 0xf0, 0x30, 0x30, 0x30, 0x30, 0xfc, 0x00, 
	/* 50 */	0x78, 0xcc, 0x0c, 0x38, 0x60, 0xcc, 0xfc, 0x00, 
	/* 51 */	0x78, 0xcc, 0x0c, 0x38, 0x0c, 0xcc, 0x78, 0x00, 
	/* 52 */	0x1c, 0x3c, 0x6c, 0xcc, 0xfe, 0x0c, 0x0c, 0x00, 
	/* 53 */	0xfc, 0xc0, 0xf8, 0x0c, 0x0c, 0xcc, 0x78, 0x00, 
	/* 54 */	0x38, 0x60, 0xc0, 0xf8, 0xcc, 0xcc, 0x78, 0x00, 
	/* 55 */	0xfc, 0xcc, 0x0c, 0x18, 0x30, 0x60, 0x60, 0x00, 
	/* 56 */	0x78, 0xcc, 0xcc, 0x78, 0xcc, 0xcc, 0x78, 0x00, 
	/* 57 */	0x78, 0xcc, 0xcc, 0x7c, 0x0c, 0x18, 0x70, 0x00, 
	/* 58 */	0x00, 0x00, 0x30, 0x30, 0x00, 0x30, 0x30, 0x00, 
	/* 59 */	0x00, 0x00, 0x30, 0x30, 0x00, 0x70, 0x30, 0x60, 
	/* 60 */	0x18, 0x30, 0x60, 0xc0, 0x60, 0x30, 0x18, 0x00, 
	/* 61 */	0x00, 0x00, 0xfc, 0x00, 0xfc, 0x00, 0x00, 0x00, 
	/* 62 */	0x60, 0x30, 0x18, 0x0c, 0x18, 0x30, 0x60, 0x00, 
	/* 63 */	0x78, 0xcc, 0x0c, 0x18, 0x30, 0x00, 0x30, 0x00, 
	/* 64 */	0x7c, 0xc6, 0xde, 0xde, 0xde, 0xc0, 0x78, 0x00, 
	/* 65 */	0x30, 0x78, 0xcc, 0xcc, 0xfc, 0xcc, 0xcc, 0x00, 
	/* 66 */	0xfc, 0x66, 0x66, 0x7c, 0x66, 0x66, 0xfc, 0x00, 
	/* 67 */	0x3c, 0x66, 0xc0, 0xc0, 0xc0, 0x66, 0x3c, 0x00, 
	/* 68 */	0xfc, 0x6c, 0x66, 0x66, 0x66, 0x6c, 0xfc, 0x00, 
	/* 69 */	0xfe, 0x62, 0x68, 0x78, 0x68, 0x62, 0xfe, 0x00, 
	/* 70 */	0xfe, 0x62, 0x68, 0x78, 0x68, 0x60, 0xf0, 0x00, 
	/* 71 */	0x3c, 0x66, 0xc0, 0xc0, 0xce, 0x66, 0x3e, 0x00, 
	/* 72 */	0xcc, 0xcc, 0xcc, 0xfc, 0xcc, 0xcc, 0xcc, 0x00, 
	/* 73 */	0x78, 0x30, 0x30, 0x30, 0x30, 0x30, 0x78, 0x00, 
	/* 74 */	0x1e, 0x0c, 0x0c, 0x0c, 0xcc, 0xcc, 0x78, 0x00, 
	/* 75 */	0xe6, 0x66, 0x6c, 0x78, 0x6c, 0x66, 0xe6, 0x00, 
	/* 76 */	0xf0, 0x60, 0x60, 0x60, 0x62, 0x66, 0xfe, 0x00, 
	/* 77 */	0xc6, 0xee, 0xfe, 0xd6, 0xc6, 0xc6, 0xc6, 0x00, 
	/* 78 */	0xc6, 0xe6, 0xf6, 0xde, 0xce, 0xc6, 0xc6, 0x00, 
	/* 79 */	0x38, 0x6c, 0xc6, 0xc6, 0xc6, 0x6c, 0x38, 0x00, 
	/* 80 */	0xfc, 0x66, 0x66, 0x7c, 0x60, 0x60, 0xf0, 0x00, 
	/* 81 */	0x78, 0xcc, 0xcc, 0xcc, 0xdc, 0x78, 0x1c, 0x00, 
	/* 82 */	0xfc, 0x66, 0x66, 0x7c, 0x78, 0x6c, 0xe6, 0x00, 
	/* 83 */	0x78, 0xcc, 0xe0, 0x38, 0x1c, 0xcc, 0x78, 0x00, 
	/* 84 */	0xfc, 0xb4, 0x30, 0x30, 0x30, 0x30, 0x78, 0x00, 
	/* 85 */	0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0xfc, 0x00, 
	/* 86 */	0xcc, 0xcc, 0xcc, 0xcc, 0xcc, 0x78, 0x30, 0x00, 
	/* 87 */	0xc6, 0xc6, 0xc6, 0xd6, 0xfe, 0xee, 0xc6, 0x00, 
	/* 88 */	0xc6, 0xc6, 0x6c, 0x38, 0x6c, 0xc6, 0xc6, 0x00, 
	/* 89 */	0xcc, 0xcc, 0xcc, 0x78, 0x30, 0x30, 0x78, 0x00, 
	/* 90 */	0xfe, 0xcc, 0x98, 0x30, 0x62, 0xc6, 0xfe, 0x00, 
	/* 91 */	0x78, 0x60, 0x60, 0x60, 0x60, 0x60, 0x78, 0x00, 
	/* 92 */	0xc0, 0x60, 0x30, 0x18, 0x0c, 0x06, 0x02, 0x00, 
	/* 93 */	0x78, 0x18, 0x18, 0x18, 0x18, 0x18, 0x78, 0x00, 
	/* 94 */	0x10, 0x38, 0x6c, 0xc6, 0x00, 0x00, 0x00, 0x00, 
	/* 95 */	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 
	/* 96 */	0x30, 0x30, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 
	/* 97 */	0x00, 0x00, 0x78, 0x0c, 0x7c, 0xcc, 0x76, 0x00, 
	/* 98 */	0xe0, 0x60, 0x7c, 0x66, 0x66, 0x66, 0xbc, 0x00, 
	/* 99 */	0x00, 0x00, 0x78, 0xcc, 0xc0, 0xcc, 0x78, 0x00, 
	/* 100 */	0x1c, 0x0c, 0x0c, 0x7c, 0xcc, 0xcc, 0x76, 0x00, 
	/* 101 */	0x00, 0x00, 0x78, 0xcc, 0xfc, 0xc0, 0x78, 0x00, 
	/* 102 */	0x38, 0x6c, 0x60, 0xf0, 0x60, 0x60, 0xf0, 0x00, 
	/* 103 */	0x00, 0x00, 0x76, 0xcc, 0xcc, 0x7c, 0x0c, 0xf8, 
	/* 104 */	0xe0, 0x60, 0x6c, 0x76, 0x66, 0x66, 0xe6, 0x00, 
	/* 105 */	0x30, 0x00, 0x70, 0x30, 0x30, 0x30, 0x78, 0x00, 
	/* 106 */	0x18, 0x00, 0x78, 0x18, 0x18, 0x18, 0xd8, 0x70, 
	/* 107 */	0xe0, 0x60, 0x66, 0x6c, 0x78, 0x6c, 0xe6, 0x00, 
	/* 108 */	0x70, 0x30, 0x30, 0x30, 0x30, 0x30, 0x78, 0x00, 
	/* 109 */	0x00, 0x00, 0xec, 0xfe, 0xd6, 0xc6, 0xc6, 0x00, 
	/* 110 */	0x00, 0x00, 0xf8, 0xcc, 0xcc, 0xcc, 0xcc, 0x00, 
	/* 111 */	0x00, 0x00, 0x78, 0xcc, 0xcc, 0xcc, 0x78, 0x00, 
	/* 112 */	0x00, 0x00, 0xdc, 0x66, 0x66, 0x7c, 0x60, 0xf0, 
	/* 113 */	0x00, 0x00, 0x76, 0xcc, 0xcc, 0x7c, 0x0c, 0x1e, 
	/* 114 */	0x00, 0x00, 0xd8, 0x6c, 0x6c, 0x60, 0xf0, 0x00, 
	/* 115 */	0x00, 0x00, 0x7c, 0xc0, 0x78, 0x0c, 0xf8, 0x00, 
	/* 116 */	0x10, 0x30, 0x7c, 0x30, 0x30, 0x34, 0x18, 0x00, 
	/* 117 */	0x00, 0x00, 0xcc, 0xcc, 0xcc, 0xcc, 0x76, 0x00, 
	/* 118 */	0x00, 0x00, 0xcc, 0xcc, 0xcc, 0x78, 0x30, 0x00, 
	/* 119 */	0x00, 0x00, 0xc6, 0xc6, 0xd6, 0xfe, 0x6c, 0x00, 
	/* 120 */	0x00, 0x00, 0xc6, 0x6c, 0x38, 0x6c, 0xc6, 0x00, 
	/* 121 */	0x00, 0x00, 0xcc, 0xcc, 0xcc, 0x7c, 0x0c, 0xf8, 
	/* 122 */	0x00, 0x00, 0xfc, 0x98, 0x30, 0x64, 0xfc, 0x00, 
	/* 123 */	0x1c, 0x30, 0x30, 0xe0, 0x30, 0x30, 0x1c, 0x00, 
	/* 124 */	0x18, 0x18, 0x18, 0x00, 0x18, 0x18, 0x18, 0x00, 
	/* 125 */	0xe0, 0x30, 0x30, 0x1c, 0x30, 0x30, 0xe0, 0x00, 
	/* 126 */	0x76, 0xdc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	/* 127 */	0x10, 0x38, 0x6c, 0xc6, 0xc6, 0xc6, 0xfe, 0x00
};
#endif

Console console;
