/* -*- c++ -*- ***************************************************************/
/* Echtzeitsysteme                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*                            C o n s o l e                                  */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/* Printing text on the GBA screen                                           */
/*---------------------------------------------------------------------------*/

#ifndef __devices_console_h__
#define __devices_console_h__

#include "infra/memory_mapped.h"
#include "infra/types.h"

/**
 * \brief Printing text on the GBA screen
 **/
class Console {

  Memory_Mapped< ezstubs_uint16,0x4000000 > DISPCNT;
  Memory_Mapped< ezstubs_uint16,0x400000C > BG2CNT;
  Memory_Mapped< ezstubs_uint16,0x400001A > BG2VOFS;

  /**
   * \brief Initialization
   **/
  void init();
  void crlf();

  unsigned short xPos;
  unsigned short yPos;
  unsigned short scrollPos;
  unsigned short *textMap;
  unsigned char bgColor;
  unsigned char textColor;
  unsigned char mode;
  unsigned short orMask;

protected:

  enum {
    WIDTH   = 30,
    VWIDTH  = 32,
    HEIGHT  = 20,
    VHEIGHT = 64
  };

public:

  enum DispProp	{
    DP_REVERSE
  };

  Console() {
    init();
  }

  /**
   * \brief Print a single character on the GBA screen
   * \param c The character to be printed
   **/
  void print(char c);

  /**
   * \brief Emtpy the GBA screen
   **/
  void clear();

  void setProperty(DispProp property, short val);

  /**
   * \brief Set the cursor position
   * \param x The X-position
   * \param y The Y-position
   **/
  void setPosition(unsigned char x, unsigned char y);

  /**
   * \brief Get the cursor position
   * \param x The X-position
   * \param y The Y-position
   **/
  void getPosition(unsigned char *x, unsigned char *y);

  /**
   * \brief Scroll down the screen
   * \param val The number of lines to scroll down
   **/
  void scroll(short val);
};

extern Console console;

#endif /* __devices_console_h__ */
